<section>
  <header>
    <h1><%= t('DeadJobs') %></h1>
    <% if @dead.size > 0 && @total_size > @count %>
      <%= erb :_paging, locals: { url: "#{root_path}morgue" } %>
    <% end %>
    <%= filtering('morgue') %>
  </header>

  <% if @dead.size > 0 %>
    <form action="<%= root_path %>morgue" method="post">
      <%= csrf_tag %>
      <div class="table_container">
        <table>
          <thead>
            <tr>
              <th>
                <label>
                  <input type="checkbox" class="check_all check-all-items">
                </label>
              </th>
              <th><%= t('LastRetry') %></th>
              <th><%= t('Queue') %></th>
              <th><%= t('Job') %></th>
              <th><%= t('Arguments') %></th>
              <th><%= t('Error') %></th>
            </tr>
          </thead>
          <% @dead.each do |entry| %>
            <tr>
              <td>
                <label>
                  <input type="checkbox" name="key[]" value="<%= job_params(entry.item, entry.score) %>" class="shift_clickable select-item-checkbox">
                </label>
              </td>
              <td>
                <a href="<%= root_path %>morgue/<%= job_params(entry.item, entry.score) %>"><%= relative_time(entry.at) %></a>
              </td>
              <td>
                <a href="<%= root_path %>queues/<%= CGI.escape(entry.queue) if entry.queue %>"><%= h entry.queue %></a>
              </td>
              <td>
                <%= entry.display_class %>
                <%= display_tags(entry, "morgue") %>
              </td>
              <td>
                <div class="args"><%= display_args(entry.display_args) %></div>
              </td>
              <td>
                <% if entry.error? %>
                  <div><a href="<%= root_path %>morgue/<%= job_params(entry.item, entry.score) %>"><%= h truncate("#{entry['error_class']}: #{entry['error_message']}", 200) %></a></div>
                <% end %>
              </td>
            </tr>
          <% end %>
        </table>
      </div>
      <div class="buttons-row">
        <input class="btn btn-primary pull-left flip" type="submit" name="retry" value="<%= t('RetryNow') %>">
        <input class="btn btn-danger pull-left flip" type="submit" name="delete" value="<%= t('Delete') %>">
        <button class="btn btn-primary bulk-action-buttons bulk-lead-button" type="submit" name="action" value="retry_all" formaction="<%= "#{root_path}morgue/all/retry" %>"><%= t('RetryAll') %></button>
        <button class="btn btn-danger bulk-action-buttons" type="submit" name="action" value="delete_all" formaction="<%= "#{root_path}morgue/all/delete" %>" data-confirm="<%= t('AreYouSure') %>"><%= t('DeleteAll') %></button>
      </div>
    </form>

  <% else %>
    <div class="alert alert-success"><%= t('NoDeadJobsFound') %></div>
  <% end %>
</section>
