<footer>
  <div class="container">
    <ul class="nav">
      <li>
        <p><b><%= product_version %></b></p>
      </li>
      <li>
        <p title="<%= redis_url %>"><%= redis_url %></p>
      </li>
      <li>
        <p id="serverUtcTime"><%= server_utc_time %></p>
      </li>
      <li>
        <p><a rel="help" href="https://sidekiq.org/wiki">docs</a></p>
      </li>
      <li>
        <form id="locale-form" action="<%= root_path %>change_locale" method="post">
          <%= csrf_tag %>
          <select id="locale-select" aria-label="<%= t("Language") %>" class="form-control" name="locale">
            <% available_locales.each do |locale_option| %>
              <% if locale_option == locale %>
                <option selected value="<%= locale_option %>"><%= language_name(locale_option) %></option>
              <% else %>
                <option value="<%= locale_option %>"><%= language_name(locale_option) %></option>
              <% end %>
            <% end %>
          </select>
        </form>
      </li>
    </ul>
  </div>
</footer>
