<% if pollable? %>
  <a href="#" class="btn live-poll live-poll-start btn-primary"><%= t('LivePoll') %></a>
  <a href="#" class="btn live-poll live-poll-stop btn-danger"><%= t('StopPolling') %></a>
<% end %>
