<!doctype html>
<html dir="<%= text_direction %>" lang="<%= locale %>">
  <head>
    <title><%= environment_title_prefix %><%= Sidekiq::NAME %></title>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width,initial-scale=1.0">
    <meta name="color-scheme" content="light dark">
    <link href="<%= root_path %>stylesheets/style.css" media="screen" rel="stylesheet" type="text/css" nonce="<%= csp_nonce %>">
    <link rel="apple-touch-icon" href="<%= root_path %>images/apple-touch-icon.png">
    <link rel="shortcut icon" type="image/ico" href="<%= root_path %>images/favicon.ico">
    <script type="text/javascript" src="<%= root_path %>javascripts/application.js" nonce="<%= csp_nonce %>"></script>
    <meta name="google" content="notranslate">
    <%= display_custom_head %>
  </head>
  <body class="admin" data-locale="<%= locale %>">
    <%= erb :_nav %>
    <main id="page">
      <div class="container">
        <%= erb :_summary %>
        <% if flash? %><p class="flash"><%= get_flash %></p><% end %>
        <%= yield %>
      </div>
    </main>
    <%= erb :_footer %>
  </body>
</html>
