<div class="cards-container">
  <article>
    <span id="txtProcessed" class="count" data-nwp><%= stats.processed %></span>
    <span class="desc"><%= t('Processed') %></span>
  </article>
  <article>
    <span id="txtFailed" class="count" data-nwp><%= stats.failed %></span>
    <span class="desc"><%= t('Failed') %></span>
  </article>
  <article>
    <a href="<%= root_path %>busy">
      <span id="txtBusy" class="count" data-nwp><%= workset.size %></span>
      <span class="desc"><%= t('Busy') %></span>
    </a>
  </article>
  <article>
    <a href="<%= root_path %>queues">
      <span id="txtEnqueued" class="count" data-nwp><%= stats.enqueued %></span>
      <span class="desc"><%= t('Enqueued') %></span>
    </a>
  </article>
  <article>
    <a href="<%= root_path %>retries">
      <span id="txtRetries" class="count" data-nwp><%= stats.retry_size %></span>
      <span class="desc"><%= t('Retries') %></span>
    </a>
  </article>
  <article>
    <a href="<%= root_path %>scheduled">
      <span id="txtScheduled" class="count" data-nwp><%= stats.scheduled_size %></span>
      <span class="desc"><%= t('Scheduled') %></span>
    </a>
  </article>
  <article>
    <a href="<%= root_path %>morgue">
      <span id="txtDead" class="count" data-nwp><%= stats.dead_size %></span>
      <span class="desc"><%= t('Dead') %></span>
    </a>
  </article>
</div>
